import requests
from cnaas_sync.utils import get_number_of_pages
# from utils import get_number_of_pages


class NI:
    def __init__(self, urlbase, api_user, api_token):
        self.urlbase = urlbase
        self.api_user = api_user
        self.api_token = api_token
        self.headers = {"Authorization": "ApiKey {}:{}".format(self.api_user, self.api_token)}

    def get_all_rooms(self):
        res = requests.get(self.urlbase + "/api/v1/site/", headers=self.headers, verify=False)
        if not res.status_code == 200:
            print(f"Failed to fetch devices from NI: {res.status_code}")
            return False
        print("NI Rooms:")
        for room in res.json()["objects"]:
            print(f"Room {room['node_name']} found in NI!")
        return res.json()["objects"]

    def get_all_devices(self, limit=1000):
        # offset should be divisible by limit or the opposite otherwise raise error
        offset = 0
        res = requests.get(self.urlbase + f"/api/v1/switch/?limit={limit}&offset={offset}", headers=self.headers,
                           verify=False)
        if res.status_code != 200:
            print(f"Failed to fetch devices from NI: {res.status_code}")
            return False

        data_json = res.json()
        total_count = int(data_json["meta"]["total_count"])
        num_pages = get_number_of_pages(total_count, limit)
        switches = res.json()["objects"]
        for _ in range(1, num_pages):
            offset += limit
            res = requests.get(self.urlbase + f"/api/v1/switch/?limit={limit}&offset={offset}", headers=self.headers,
                               verify=False)
            if res.status_code == 200:
                switches.extend(res.json()["objects"])
            print("offset=", offset)

        return switches

    def add_new_managed_device(self, hostname, switches, device_type, serial_number, model, management_ip, vendor):
        data = {
            "node_meta_type": "Physical",
            "node_name": f"{hostname}",
            "node_type": "/api/v1/node_type/switch/",
            "node": {
                "ip_addresses": [f"{management_ip}"],
                "model": model,
                "name": f"{hostname}",
                "noclook_auto_manage": "",
                "operational_state": "Reserved",
                "switch_type": "2",
                "serial_number": serial_number,
                "vendor": vendor,
            },
        }
        print("Adding switches")
        for device in switches:
            if device["node_name"] == hostname:
                print(f"Switch {hostname} already found in NI!")
                return False
        print(f"Adding {hostname}")
        res = requests.post(self.urlbase + "/api/v1/switch/", headers=self.headers, json=data, verify=False)
        if res.status_code != 201:
            print("Could not create device!")
            return False

    def update(self, nms_devices):
        ni_devices = self.get_all_devices(limit=1000)
        for device in nms_devices:
            print(device["hostname"])
            if device["vendor"] != "Arista":
                continue
            hostname = device["hostname"]
            serial = device["serial"]
            model = device["model"]
            management_ip = device["management_ip"]
            self.add_new_managed_device(hostname, ni_devices, "ACCESS", serial, model, management_ip, "Arista")
