import requests
from cnaas_sync.utils import get_number_of_pages
# from utils import get_number_of_pages


class NMS:
    def __init__(self, urlbase, jwttoken):
        self.urlbase = urlbase
        self.jwttoken = jwttoken

    def get_all_devices(self):
        headers = {"Authorization": "Bearer " + self.jwttoken}
        res = requests.get(self.urlbase + "/api/v1.0/devices", headers=headers, verify=False)
        if res.status_code != 200:
            print(f"Failed to fetch devices from NMS! Status Code: {res.status_code}")
            return []
        page_size = 1000
        total_count = int(res.headers["X-Total-Count"])
        number_pages = get_number_of_pages(total_count, page_size)
        accumulated_res = []
        # Todo simulatnously call all the page_size concurently with asyncio/
        for page in range(number_pages):
            res = requests.get(
                self.urlbase + f"/api/v1.0/devices?per_page={page_size}&page=" + str(page), headers=headers,
                verify=False
            )
            if "devices" not in res.json()["data"]:
                print(f"Failed to fetch devices from NMS! Status Code: {res.status_code}")
                return None
            accumulated_res += res.json()["data"]["devices"]
        return accumulated_res

    def get_device_type(self, hostname):
        headers = {"Authorization": "Bearer " + self.jwttoken}
        res = requests.get(self.urlbase + "/api/v1.0/device/" + hostname, headers=headers, verify=False)
        if res.status_code != 200:
            print(f"Failed to fetch devices from NMS! Status Code: {res.status_code}")
            return None
        return res.json()["data"]["devices"][0]["device_type"]
